//-----------------------------------------------------------------
// SX3000EVO Firmware di test
// Versione : 1.0
// Area SX
// http://www.areasx.com
// Note     :
//-----------------------------------------------------------------

//-----------------------------------------------------------------
// ---- Configurazione del firmware ----
// Rimuovere il commento davanti ad una delle righe seguenti per
// abiltare il test sulla rispettiva sezione hardware
// Il firmware  scritto al solo scopo di test e per fornire una
// base di partenza per lo sviluppo. E' opportuno non abilitare
// pi di un test alla volta
//-----------------------------------------------------------------
//#define DEBUG_ANALOG      // Lettura degli ingressi analogici
#define DEBUG_XBEE         // Test del modulo XBEE (se montato)
//#define DEBUG_SERIAL2     // Test delle Seriale 2
//#define DEBUG_SERIAL1     // Test delle Seriale 2
#define DEBUG_LED         // Test dei led presenti sulla scheda
#define DEBUG_SHT75         // Test SHT75 (solo con libreria)
//-----------------------------------------------------------------
// Buffer delle seriali
//-----------------------------------------------------------------
#define EINBUFSIZE 63
#define EOUTBUFSIZE 63
#define DINBUFSIZE 63
#define DOUTBUFSIZE 63
#define CINBUFSIZE 63
#define COUTBUFSIZE 63

//-----------------------------------------------------------------
// Macro Hardware
//-----------------------------------------------------------------
#define LED1_ON       BitWrPortI(PBDR, &PBDRShadow, 1, 2);
#define LED1_OFF      BitWrPortI(PBDR, &PBDRShadow, 0, 2);
#define LED2_ON       BitWrPortI(PEDR, &PEDRShadow, 1, 5);
#define LED2_OFF      BitWrPortI(PEDR, &PEDRShadow, 0, 5);
#define LED3_ON       BitWrPortI(PEDR, &PEDRShadow, 1, 7);
#define LED3_OFF      BitWrPortI(PEDR, &PEDRShadow, 0, 7);
#define BUTTON_PRESS  (BitRdPortI(PBDR,0)==1)


//Define AD
#define AD_DATA_DOWN            BitWrPortI(PBDR, &PBDRShadow, 0, 5)
#define AD_DATA_UP              BitWrPortI(PBDR, &PBDRShadow, 1, 5)
#define AD_DATAIN_UP            BitRdPortI(PBDR,7)
#define AD_CS_DOWN              BitWrPortI(PBDR, &PBDRShadow, 0, 3)
#define AD_CS_UP                BitWrPortI(PBDR, &PBDRShadow, 1, 3)
#define AD_CLOCK_UP             BitWrPortI(PBDR, &PBDRShadow, 1, 4)
#define AD_CLOCK_DOWN           BitWrPortI(PBDR, &PBDRShadow, 0, 4)
#define AD_CLOCK_STROBE         AD_CLOCK_UP;AD_CLOCK_DOWN



#ifdef DEBUG_SHT75
	//Define sensore SHT75
	#define SHT_CLOCK_BIT         1
	#define SHT_DATA_BIT          0
	#define SHT_DATA_LINE_IN      BitWrPortI(PEDDR,&PEDDRShadow,0,SHT_DATA_BIT)
	#define SHT_DATA_LINE_OUT     BitWrPortI(PEDDR,&PEDDRShadow,1,SHT_DATA_BIT)
	#define SHT_DATA_LINE_LOW     SHT_DATA_LINE_OUT;BitWrPortI(PEDR,&PEDRShadow,0,SHT_DATA_BIT)
	#define SHT_DATA_LINE_HIGH    SHT_DATA_LINE_IN
	#define SHT_DATA_LINE_READ    BitRdPortI(PEDR,SHT_DATA_BIT)
	#define SHT_CLOCK_LINE_IN     BitWrPortI(PEDDR,&PEDDRShadow,0,SHT_CLOCK_BIT)
	#define SHT_CLOCK_LINE_OUT    BitWrPortI(PEDDR,&PEDDRShadow,1,SHT_CLOCK_BIT)
	#define SHT_CLOCK_LINE_LOW    SHT_CLOCK_LINE_OUT;BitWrPortI(PEDR,&PEDRShadow,0,SHT_CLOCK_BIT)
	#define SHT_CLOCK_LINE_HIGH   SHT_CLOCK_LINE_IN
#endif

//-----------------------------------------------------------------
// Prototipi delle funzioni
//-----------------------------------------------------------------
void msDelay(unsigned int);
void BoardInit(void);




//VARIABILI GLOBALI
char XbeeRXbuff[100];
float temperature, humidity;

#ifdef DEBUG_SHT75
  #use "SHT75.LIB"
#endif
//--------------------------------------------------------
// Ritardo bloccante
// Riceve      : ams - numero di millisecondi da ritardare
// Restituisce : -
// Note        :
//--------------------------------------------------------
nodebug
void msDelay(unsigned int ams)
{
  auto unsigned long t0;
  for(t0=MS_TIMER; MS_TIMER<ams+t0;);
}

//--------------------------------------------------------
// Inizializza i port del modulo RCM3700
// Riceve      : -
// Restituisce : -
// Note        :
//--------------------------------------------------------
void BoardInit() {

  // Inizializzazione port PB0 in ingresso (pulsante)
  BitWrPortI(PBDDR, &PBDDRShadow, 0,0);

  // Inizializzazione port PB2 (Led 1)
  BitWrPortI(PBDDR, &PBDDRShadow, 1,2);

  // Inizializzazione port PB3 PB4 PB5 PB7 (Analogica)
  BitWrPortI(PBDDR, &PBDDRShadow, 1,3);
  BitWrPortI(PBDDR, &PBDDRShadow, 1,4);
  BitWrPortI(PBDDR, &PBDDRShadow, 1,5);
  BitWrPortI(PBDDR, &PBDDRShadow, 0,7);

  // Inizializzazione port PE5 e PE7 (Led 2 e 3)
  WrPortI(PEFR, &PEFRShadow,0 );
  BitWrPortI(PEDDR, &PEDDRShadow, 1,5);
  BitWrPortI(PEDDR, &PEDDRShadow, 1,7);

  // Inizializzazione port PG6 (Seriale E)
  BitWrPortI(PGFR, &PGFRShadow, 1,6);

  // Inizializzazione seriale su PC2-PC3 (Ser C)
  BitWrPortI(PCFR, &PCFRShadow, 1,2);
  // Inizializzazione seriale su PC0-PC1 (Ser D)
  BitWrPortI(PCFR, &PCFRShadow, 1,0);

  // Inizializzazione port PF0 PF1 per il controllo di flusso
  WrPortI(PFFR, &PFFRShadow, 0x00);
  WrPortI(PFDCR, &PFDCRShadow, 0x00);
  BitWrPortI(PFDDR, &PFDDRShadow, 1,0);
  BitWrPortI(PFDDR, &PFDDRShadow, 0,1);

  //SHT75 su J2
    BitWrPortI(PEFR, &PEFRShadow, 0,SHT_DATA_BIT); // setta come I/0
    BitWrPortI(PEFR, &PEFRShadow, 0,SHT_CLOCK_BIT);
    BitWrPortI(PEDDR, &PEDDRShadow, 1,SHT_DATA_BIT); //Output
    BitWrPortI(PEDDR, &PEDDRShadow, 1,SHT_CLOCK_BIT); //Output
}



// -----------------------------------------------------------------------
// Legge un canale AD
// Riceve      : channel - numero del canale da leggere (0 o 1)
// Restituisce : numero di punti letti sul campionatore
// Note        :
// -----------------------------------------------------------------------
int ADC_Read(char channel) {
  char ADRes;
  int i;
  // Start conversion
  AD_CLOCK_DOWN;
  AD_CS_DOWN;
  // MUX Addressing
  // Start Bit
  AD_DATA_UP;
  AD_CLOCK_STROBE;
  // Single ended
  AD_DATA_UP;
  AD_CLOCK_STROBE;
  if (channel)
    AD_DATA_UP; // Canale 1 abilitato
  else
    AD_DATA_DOWN; // Canale 0 abilitato
  AD_CLOCK_STROBE;
  // Inizio ricezione dato di conversione
  ADRes=0;
  // Leading 0
  AD_CLOCK_UP;
  if (AD_DATAIN_UP) {
    #ifdef DEBUG_STRING2
      printf ("Start Bit ERRATO\n");
    #endif
    return -1;
  }
  msDelay(1);
  AD_CLOCK_DOWN;
  msDelay(1);
  for (i=0;i<8;i++) {
    AD_CLOCK_UP;
    if (!AD_DATAIN_UP)
      ADRes=ADRes<<1;
    else
      ADRes=(ADRes<<1) | 0x01;
    msDelay(1);
    AD_CLOCK_DOWN;
    msDelay(1);
  }
  // Ricezione dell'inverso
  for (i=0;i<7;i++) {
    AD_CLOCK_UP;
    msDelay(1);
    AD_CLOCK_DOWN;
    msDelay(1);
  }
  AD_CS_UP;
  #ifdef DBG_STRING2
    printf("Conversione AD %d\n", ADRes);
  #endif
  return ADRes;
}


int Send2XbeeModule(char *cmd, int cmdSize, int timeout,int CR, int WaitAnswer, int ReciveChar){
  //XBEE
  char XbeeTXbuff[100];
  unsigned long t0;
  int res;

  serDrdFlush(); // Pulizia buffer di ricezione e trasmissione
  serDwrFlush();
  XbeeTXbuff[0]=0;
  XbeeRXbuff[0]=0;

  if(CR) {
    sprintf(XbeeTXbuff,"%s\r",cmd);
    serDwrite(XbeeTXbuff,(strlen(cmd)+1));
  } else {
    sprintf(XbeeTXbuff,"%s",cmd);
    serDwrite(XbeeTXbuff,strlen(cmd));
  }
  if(WaitAnswer){
    msDelay(timeout);

    t0=MS_TIMER;
    while(serCpeek()==-1 && MS_TIMER<timeout+t0){ }
    res=serDread(XbeeRXbuff,ReciveChar,timeout);
    #ifdef DBG_STRING
        printf("CH da xbee %d\n",res);
    #endif
    if (res>0) {
      printf("Il modulo ha risposto! ch %d \n", res);
      XbeeRXbuff[res]=0;
      printf("-->%s\n", XbeeRXbuff);
      return res;
    } else {
      printf("Nessuna risposta dal modulo XBEE\n");
      return 0;
    }
  } else
    return 0;
}

// -----------------------------------
// Loop Principale
// -----------------------------------
main () {
  char ch;
  int i;
  int rec;

  BoardInit();

  // Inizializza la seriale a 9600 baud
  serCopen(9600);

  // Inizializza la seconda seriale a 9600 baud
  serEopen(9600);

  // Inizializza XBEE a 9600 baud
  serDopen(9600);



  printf ("Inizio test scheda SX3000EVO. Versione 1.0\n");

  while(1) {
      #ifdef DEBUG_SHT75
        //LEGGE LA TEMPERATURA E L'UMIDITA'
        if (SHT_ReadHumidity() && SHT_ReadTemperature()) {
  
            printf("\n---------------\n");
            printf("Temperatura: %.2f C\n", temperature);
            printf("Umidita': %.2f %%\n", humidity);
            printf("\n---------------\n");
        }
      #endif

    //-----------------------------------------------------------------
    // Debug Analogica
    // Legge dal Canale 0 dell'analogica, in modalit single ended
    // Esegue una lettura ogni 5 secondi
    //-----------------------------------------------------------------
    #ifdef DEBUG_ANALOG
      printf("AD 0 %d\n", ADC_Read(0));
      printf("AD 1 %d\n", ADC_Read(1));
    #endif

    //-----------------------------------------------------------------
    // Debug XBEE
    // Interroga il modulo e ne ricava versione e release
    // Se viene premuto BTN1 sulla scheda invia il carattere 'M'
    // Accende il led T3 durante la trasmissione
    // Accende il led T1 quando viene premuto il pulsante BTN1
    //-----------------------------------------------------------------
    #ifdef DEBUG_XBEE
      rec=Send2XbeeModule("+++", 2, 1020,0, 1, 3);
      if(rec>0)
        printf("Il modulo XBEE ha risposto: %s\n", XbeeRXbuff);
        msDelay(500);
        rec=Send2XbeeModule("ATHV", 4, 110,1, 1, 5);
        if(rec>0)
          printf("Il modulo XBEE VERSIONE: %s\n", XbeeRXbuff);
        rec=Send2XbeeModule("ATVR", 4, 200,1, 1, 5);
        if(rec>0)
          printf("Il modulo XBEE RELEASE: %s\n", XbeeRXbuff);
      msDelay(5000);
    #endif


    //-----------------------------------------------------------------
    // Debug seriale 2
    // Replica i caratteri ricevuti sulla seriale
    // Se si preme BTN1 invia il carattere 'M' ed accede il led T1
    //-----------------------------------------------------------------
    #ifdef DEBUG_SERIAL2
      if ((ch=serEgetc())!=-1) {

        printf ("Ricevuto carattere [%c]\n",ch);

        serEputc(ch);

      }

      if (BUTTON_PRESS) {

        ch=0x4D;
        printf ("Invio carattere [%c]\n",ch);

        serEputc(ch);

          LED1_ON
          msDelay(100);
          LED1_OFF
      }
    #endif

    //-----------------------------------------------------------------
    // Debug seriale 1
    // Replica i caratteri ricevuti sulla seriale
    // Se si preme BTN1 invia il carattere 'M' ed accede il led T1
    //-----------------------------------------------------------------
    #ifdef DEBUG_SERIAL1
      if ((ch=serCgetc())!=-1) {

        printf ("Ricevuto carattere [%c]\n",ch);

        serCputc(ch);

      }

      if (BUTTON_PRESS) {

        ch=0x4D;
        printf ("Invio carattere [%c]\n",ch);

        serCputc(ch);

          LED1_ON
          msDelay(100);
          LED1_OFF
      }

    #endif


    //-----------------------------------------------------------------
    // Debug Led
    // Lampeggia alternativamente i led T1, T2 e T3
    // Se si preme BTN1 esegue un lampeggio contemporaneo
    //-----------------------------------------------------------------
    #ifdef DEBUG_LED
  		printf ("Testing...\n");
      LED1_ON
      LED2_OFF
      LED3_OFF

      msDelay(500);
      LED1_OFF
      LED2_ON
      LED3_OFF
      msDelay(500);

      LED1_OFF
      LED2_OFF
      LED3_ON
      msDelay(500);


      if (BUTTON_PRESS) {
        LED1_ON
        LED2_ON
        LED3_ON

        msDelay(100);

        LED1_OFF
        LED2_OFF
        LED3_OFF

        msDelay(100);
        LED1_ON
        LED2_ON
        LED3_ON

        msDelay(100);

        LED1_OFF
        LED2_OFF
        LED3_OFF
        msDelay(100);

      }
    #endif

  }
}